@@ -4,6 +4,7 @@
 from __future__ import absolute_import
 import pytest
 import sys
+import gc
 
 _sqlite3 = pytest.importorskip('_sqlite3')
 
@@ -11,7 +12,7 @@ pypy_only = pytest.mark.skipif('__pypy__' not in sys.builtin_module_names,
     reason="PyPy-only test")
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def con():
     con = _sqlite3.connect(':memory:')
     yield con
@@ -35,9 +36,9 @@ def test_list_ddl(con):
 
 @pypy_only
 def test_connect_takes_same_positional_args_as_Connection(con):
-    from inspect import getargspec
-    clsargs = getargspec(_sqlite3.Connection.__init__).args[1:]  # ignore self
-    conargs = getargspec(_sqlite3.connect).args
+    from inspect import getfullargspec
+    clsargs = getfullargspec(_sqlite3.Connection.__init__).args[1:]  # ignore self
+    conargs = getfullargspec(_sqlite3.connect).args
     assert clsargs == conargs
 
 def test_total_changes_after_close(con):
@@ -168,15 +169,12 @@ def test_on_conflict_rollback_executemany(con):
         pytest.fail("_sqlite3 knew nothing about the implicit ROLLBACK")
 
 def test_statement_arg_checking(con):
-    with pytest.raises(_sqlite3.Warning) as e:
+    with pytest.raises(TypeError) as e:
         con(123)
-    assert str(e.value).startswith('SQL is of wrong type. Must be string')
-    with pytest.raises(ValueError) as e:
+    with pytest.raises(TypeError) as e:
         con.execute(123)
-    assert str(e.value).startswith('operation parameter must be str')
-    with pytest.raises(ValueError) as e:
+    with pytest.raises(TypeError) as e:
         con.executemany(123, 123)
-    assert str(e.value).startswith('operation parameter must be str')
     with pytest.raises(ValueError) as e:
         con.executescript(123)
     assert str(e.value).startswith('script argument must be unicode')
@@ -218,6 +216,13 @@ def test_returning_blob_must_own_memory(con):
     val = cur.fetchone()[0]
     assert isinstance(val, bytes)
 
+def test_function_arg_str_null_char(con):
+    con.create_function("strlen", 1, lambda a: len(a))
+    cur = con.execute("select strlen(?)", ["x\0y"])
+    val = cur.fetchone()[0]
+    assert val == 3
+
+
 def test_description_after_fetchall(con):
     cur = con.cursor()
     assert cur.description is None
@@ -264,11 +269,8 @@ def test_adapter_exception(con):
     _sqlite3.register_adapter(int, cast)
     try:
         cur = con.cursor()
-        cur.execute("select ?", (4,))
-        val = cur.fetchone()[0]
-        # Adapter error is ignored, and parameter is passed as is.
-        assert val == 4
-        assert type(val) is int
+        with pytest.raises(ZeroDivisionError):
+            cur.execute("select ?", (4,))
     finally:
         del _sqlite3.adapters[(int, _sqlite3.PrepareProtocol)]
 
@@ -347,6 +349,38 @@ def test_isolation_bug():
     cur = con.cursor()
     cur.execute("create table foo(x);")
 
+def test_reset_of_shared_statement(con):
+    con = _sqlite3.connect(':memory:')
+    c0 = con.cursor()
+    c0.execute('CREATE TABLE data(n int, t int)')
+    # insert two values
+    c0.execute('INSERT INTO data(n, t) VALUES(?, ?)', (0, 1))
+    c0.execute('INSERT INTO data(n, t) VALUES(?, ?)', (1, 2))
+
+    c1 = con.execute('select * from data')
+    list(c1) # c1's statement is no longer in use afterwards
+    c2 = con.execute('select * from data')
+    # the statement between c1 and c2 is shared
+    assert c1._Cursor__statement is c2._Cursor__statement
+    val = next(c2)
+    assert val == (0, 1)
+    c1 = None # make c1 unreachable
+    gc.collect() # calling c1.__del__ used to reset c2._Cursor__statement!
+    val = next(c2)
+    assert val == (1, 2)
+    with pytest.raises(StopIteration):
+        next(c2)
+
+def test_row_index_unicode(con):
+    import sqlite3
+    con.row_factory = sqlite3.Row
+    row = con.execute("select 1 as \xff").fetchone()
+    assert row["\xff"] == 1
+    with pytest.raises(IndexError):
+        row['\u0178']
+    with pytest.raises(IndexError):
+        row['\xdf']
+
 @pytest.mark.skipif(not hasattr(_sqlite3.Connection, "backup"), reason="no backup")
 class TestBackup:
     def test_target_is_connection(self, con):
@@ -371,3 +405,34 @@ class TestBackup:
         result = con2.execute("SELECT key FROM foo ORDER BY key").fetchall()
         assert result[0][0] == 3
         assert result[1][0] == 4
+
+def test_reset_already_committed_statements_bug(con):
+    con.execute('''CREATE TABLE COMPANY
+             (ID INT PRIMARY KEY,
+             A INT);''')
+    con.execute("INSERT INTO COMPANY (ID, A) \
+          VALUES (1, 2)")
+    cursor = con.execute("SELECT id, a from COMPANY")
+    con.commit()
+    con.execute("DROP TABLE COMPANY")
+
+def test_empty_statement():
+    r = _sqlite3.connect(":memory:")
+    cur = r.cursor()
+    for sql in ["", " ", "/*comment*/"]:
+        r = cur.execute(sql)
+        assert r.description is None
+        assert cur.fetchall() == []
+
+def test_uninit_connection():
+    con = _sqlite3.Connection.__new__(_sqlite3.Connection)
+    with pytest.raises(_sqlite3.ProgrammingError):
+        con.isolation_level
+    with pytest.raises(_sqlite3.ProgrammingError):
+        con.total_changes
+    with pytest.raises(_sqlite3.ProgrammingError):
+        con.in_transaction
+    with pytest.raises(_sqlite3.ProgrammingError):
+        con.iterdump()
+    with pytest.raises(_sqlite3.ProgrammingError):
+        con.close()
