@@ -19,18 +19,6 @@ from site import USER_BASE
 from site import USER_SITE
 HAS_USER_SITE = True
 
-
-# See also: site.py/sysconfig.py
-def _get_implementation():
-    if '__pypy__' in sys.builtin_module_names:
-        return 'PyPy'
-    return 'Python'
-
-
-IMPLEMENTATION = _get_implementation()
-IMPLEMENTATION_LOWER = IMPLEMENTATION.lower()
-
-
 WINDOWS_SCHEME = {
     'purelib': '$base/Lib/site-packages',
     'platlib': '$base/Lib/site-packages',
@@ -41,34 +29,20 @@ WINDOWS_SCHEME = {
 
 INSTALL_SCHEMES = {
     'unix_prefix': {
-        'purelib': '$base/lib/' + IMPLEMENTATION_LOWER + '$py_version_short/site-packages',
-        'platlib': '$platbase/lib/' + IMPLEMENTATION_LOWER + '$py_version_short/site-packages',
-        'headers': '$base/include/' + IMPLEMENTATION_LOWER + '$py_version_short$abiflags/$dist_name',
+        'purelib': '$base/lib/$implementation_lower$py_version_short/site-packages',
+        'platlib': '$platbase/$platlibdir/$implementation_lower$py_version_short/site-packages',
+        'headers': '$base/include/$implementation_lower$py_version_short$abiflags/$dist_name',
         'scripts': '$base/bin',
         'data'   : '$base',
         },
     'unix_home': {
-        'purelib': '$base/lib/' + IMPLEMENTATION_LOWER,
-        'platlib': '$base/lib/' + IMPLEMENTATION_LOWER,
-        'headers': '$base/include/' + IMPLEMENTATION_LOWER + '/$dist_name',
+        'purelib': '$base/lib/$implementation_lower',
+        'platlib': '$base/$platlibdir/$implementation_lower',
+        'headers': '$base/include/$implementation_lower/$dist_name',
         'scripts': '$base/bin',
         'data'   : '$base',
         },
     'nt': WINDOWS_SCHEME,
-    'pypy': {
-        'purelib': '$base/site-packages',
-        'platlib': '$base/site-packages',
-        'headers': '$base/include/$dist_name',
-        'scripts': '$base/bin',
-        'data'   : '$base',
-        },
-    'pypy_nt': {
-        'purelib': '$base/site-packages',
-        'platlib': '$base/site-packages',
-        'headers': '$base/include/$dist_name',
-        'scripts': '$base/Scripts',
-        'data'   : '$base',
-        },
     }
 
 # user site schemes
@@ -76,8 +50,8 @@ if HAS_USER_SITE:
     INSTALL_SCHEMES['nt_user'] = {
         'purelib': '$usersite',
         'platlib': '$usersite',
-        'headers': '$userbase/' + IMPLEMENTATION + '$py_version_nodot/Include/$dist_name',
-        'scripts': '$userbase/' + IMPLEMENTATION + '$py_version_nodot/Scripts',
+        'headers': '$userbase/$implementation$py_version_nodot/Include/$dist_name',
+        'scripts': '$userbase/$implementation$py_version_nodot/Scripts',
         'data'   : '$userbase',
         }
 
@@ -85,7 +59,7 @@ if HAS_USER_SITE:
         'purelib': '$usersite',
         'platlib': '$usersite',
         'headers':
-            '$userbase/include/' + IMPLEMENTATION_LOWER + '$py_version_short$abiflags/$dist_name',
+            '$userbase/include/$implementation_lower$py_version_short$abiflags/$dist_name',
         'scripts': '$userbase/bin',
         'data'   : '$userbase',
         }
@@ -95,6 +69,11 @@ if HAS_USER_SITE:
 # and to SCHEME_KEYS here.
 SCHEME_KEYS = ('purelib', 'platlib', 'headers', 'scripts', 'data')
 
+def _get_implementation():
+    if sys.implementation.name == 'pypy':
+        return 'PyPy'
+    return 'Python'
+
 
 class install(Command):
 
@@ -324,7 +303,9 @@ class install(Command):
                             'sys_exec_prefix': exec_prefix,
                             'exec_prefix': exec_prefix,
                             'abiflags': abiflags,
-                            'platlibdir': getattr(sys, 'platlibdir', 'lib'),
+                            'platlibdir': sys.platlibdir,
+                            'implementation_lower': _get_implementation().lower(),
+                            'implementation': _get_implementation(),
                            }
 
         if HAS_USER_SITE:
@@ -481,12 +462,6 @@ class install(Command):
     def select_scheme(self, name):
         """Sets the install directories by applying the install schemes."""
         # it's the caller's problem if they supply a bad name!
-        if (hasattr(sys, 'pypy_version_info') and
-                not name.endswith(('_user', '_home'))):
-            if os.name == 'nt':
-                name = 'pypy_nt'
-            else:
-                name = 'pypy'
         scheme = INSTALL_SCHEMES[name]
         for key in SCHEME_KEYS:
             attrname = 'install_' + key
